#property copyright "Public domain software by Ralph Ronnquist"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Brown
#property indicator_color2 Yellow
#property indicator_color3 Blue
#property indicator_color4 Red

#property indicator_minimum -100
#property indicator_maximum 300


extern datetime since = D'2008.01.08 22:31';
extern int fontsize = 10;
extern string font = "SanSerif";
extern color fontcolor = Gold;
extern int x = 650;
extern int y = 16;

datetime until;

double maxbalance[];
double balance[];
double roi[];
double dd[];

int trades[10000];
int tc = 0;

double first_balance = 0;
double symbol_point_value;

void addTicket(int ticket)
{
    datetime opened = OrderOpenTime();

    int lo = 0;
    int hi = tc;
    while ( hi > lo ) {
        int m = ( lo + hi ) / 2;
        if ( ticket == trades[ m ] )
            return;
        OrderSelect( trades[ m ], SELECT_BY_TICKET, MODE_HISTORY );
        if ( OrderOpenTime() < opened ) {
            lo = m + 1;
        } else {
            hi = m;
        }
    }
    for ( int i = tc; i > lo; i-- ) {
        trades[ i ] = trades[ i-1 ];
    }
    trades[ lo ] = ticket;
    tc += 1;
}

void addHistory()
{
    if ( tc == OrdersHistoryTotal() )
        return;
    tc = 0;
    first_balance = AccountBalance();

    for ( int i = OrdersHistoryTotal() - 1; i >= 0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS, MODE_HISTORY ) )
            continue;
        if ( OrderType() == OP_BUY || OrderType() == OP_SELL ) {
            if ( OrderOpenTime() >= since )
                first_balance -= OrderProfit() + OrderSwap();
                addTicket( OrderTicket() );
        }
    }
}

int init()
{
    SetIndexBuffer( 0, maxbalance );
    SetIndexStyle( 0, DRAW_NONE );
    SetIndexLabel( 0, "max balance" );
    SetIndexBuffer( 1, balance );
    SetIndexStyle( 1, DRAW_NONE );
    SetIndexLabel( 1, "max balance" );
    SetIndexBuffer( 2, roi );
    SetIndexStyle( 2, DRAW_HISTOGRAM );
    SetIndexLabel( 2, "est ROI %" );
    SetIndexBuffer( 3, dd );
    SetIndexStyle( 3, DRAW_HISTOGRAM );
    SetIndexLabel( 3, "max DD %" );

    SetLevelValue( 0, 0 );
    
    tc = 0;
    addHistory();
        
    string date = TimeToStr( since );
    until = StrToTime( ( StrToInteger( StringSubstr( date, 0, 4 ) ) + 1 ) +
                       StringSubstr( date, 4 ) );

    symbol_point_value = MarketInfo( Symbol(), MODE_TICKVALUE ) /
        MarketInfo( Symbol(), MODE_TICKSIZE );

    return(0);
}

int deinit()
{
    return(0);
}

int start()
{
    addHistory();

    for ( int b = Bars - IndicatorCounted(); b >= 0; b-- ) {
        indicate( b );
    }
    drawLine( 0, "max Bal: $" + DoubleToStr( maxbalance[ 0 ], 2 ) );
    drawLine( 1, "est ROI: " + DoubleToStr( roi[ 0 ], 1 ) + "%" );
    drawLine( 2, "max DD: " + DoubleToStr( -dd[ 0 ], 1 ) + "%" );
    double v0 = MathRound( roi[ 0 ] * 10 ) / 10.0;
    double v1 = MathRound( dd[ 0 ] * 10 ) / 10.0;
    drawLine( 3, "Ratio: " + DoubleToStr( - v0 / v1, 1 ) + ":1" );
    return(0);
}

double closed(int bar)
{
    double total = 0;
    if ( bar >= Bars )
        return( 0 );
    datetime t = Time[ bar ];
    if ( t < since )
        return( 0 );
    for ( int i = 0; i < tc; i++ ) {
        if ( ! OrderSelect( trades[ i ], SELECT_BY_TICKET, MODE_HISTORY ) )
            continue;
        if ( OrderCloseTime() < t )
            continue;
        if ( OrderOpenTime() >= t + Period() * 60 ) {
            break;
        }
        if ( bar > 0 )
            if ( OrderCloseTime() >= Time[ bar-1 ] )
                continue;
        total += OrderProfit() + OrderSwap();
    }
    return( total );
}

double maxOrderDrawDown(int unto)
{
    int from = iBarShift( Symbol(), Period(), OrderOpenTime(), false );
    double price = OrderOpenPrice();
    double value = MarketInfo( OrderSymbol(), MODE_TICKVALUE ) /
        MarketInfo( OrderSymbol(), MODE_TICKSIZE );
    if ( OrderType() == OP_BUY ) {
        double lo = iLow( OrderSymbol(), Period(),
            iLowest( OrderSymbol(), Period(), MODE_LOW, from - unto, unto) );
        return( MathMax( 0, price - lo ) * value );
    }
    if ( OrderType() == OP_SELL ) {
        double hi = iHigh( OrderSymbol(), Period(),
            iHighest( OrderSymbol(), Period(), MODE_HIGH, from - unto, unto) );
        return( MathMax( 0, hi - price ) * value );
    }
    return( 0 );
}

double maxDrawDown(int bar)
{
    double total = 0;
    if ( bar >= Bars )
        return( 0 );
    datetime t = Time[ bar ];
    if ( t < since )
        return( 0 );
    for ( int i = 0; i < tc; i++ ) {
        if ( ! OrderSelect( trades[ i ], SELECT_BY_TICKET, MODE_HISTORY ) )
            continue;
        if ( OrderOpenTime() >= t )
            break;
        if ( OrderCloseTime() < Time[ bar ] + Period() * 60 )
            continue;
        total += maxOrderDrawDown( bar );
    }
    for ( i = OrdersTotal() - 1; i >= 0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS ) )
            continue;
        if ( OrderOpenTime() > t )
            continue;
        total += maxOrderDrawDown( bar );
    }
    return( - total );// * symbol_point_value );
}

void indicate(int bar)
{
    if ( bar + 1 >= Bars ) {
        maxbalance[ bar ] = first_balance;
        balance[ bar ] = first_balance;
    } else if ( Time[ bar ] < since ) {
        maxbalance[ bar ] = first_balance;
        balance[ bar ] = first_balance;
    } else {
        balance[ bar ] = balance[ bar+1 ] + closed( bar+1 );
        maxbalance[ bar ] = MathMax( maxbalance[ bar+1 ], balance[ bar ] );
        if ( Time[ bar ] > since ) {
            double extrapolation = ( 1.0 * Time[ bar ] - since ) / ( until - since );
            roi[ bar ] = ( balance[ bar ] / first_balance - 1 ) / extrapolation * 100.0;
            //roi[ bar ] = first_balance * ( 1 + 0.2 * extrapolation );
        } else {
            roi[ bar ] = 0;
        }
        if ( bar+1 < Bars )
            dd[ bar ] = MathMin( dd[ bar+ 1], 100.0 * maxDrawDown( bar ) / maxbalance[ bar+1 ] );
        else
            dd[ bar ] = 0;
    }
}

void drawLine(int line,string text)
{
    string name = "line " + line;
    if ( ObjectFind( name ) < 0 ) {
        ObjectCreate( name, OBJ_LABEL, 0, 0, 0 );
    }
    ObjectSet( name, OBJPROP_XDISTANCE, x );
    ObjectSet( name, OBJPROP_YDISTANCE, y + ( fontsize * 1.5 ) * ( line + 1 ) );
    ObjectSetText( name, text, fontsize, font, fontcolor );
}